"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginLogger = exports.ConsoleLogger = void 0;
const winston = require("winston");
class ConsoleLogger {
    constructor(config) {
        const transports = [];
        if (config.console) {
            const options = {
                exitOnError: false,
            };
            if (config.console.level) {
                options.level = config.console.level;
            }
            const formatters = [];
            if (config.console.colorize) {
                formatters.push(winston.format.colorize());
            }
            if (config.console.json) {
                formatters.push(winston.format.json());
            }
            else {
                formatters.push(winston.format.align());
                formatters.push(winston.format.simple());
            }
            options.format = winston.format.combine(...formatters);
            transports.push(new winston.transports.Console(options));
        }
        this.logger = winston.createLogger({
            level: config.level,
            exitOnError: false,
            transports: transports,
        });
        this.errorWriter = {
            write: (message) => {
                this.logger.error(message);
            },
        };
        this.debugWriter = {
            write: (message) => {
                this.logger.debug(message);
            },
        };
    }
    logEntry(level, message, ...optionalParams) {
        const meta = {};
        if (optionalParams) {
            for (let n = 0; n < optionalParams.length; n += 2) {
                const key = optionalParams[n];
                const value = optionalParams[n + 1];
                if (key !== null && value !== null) {
                    meta[key] = value;
                }
            }
        }
        this.logger.log(level, message, meta);
    }
    debug(message, ...optionalParams) {
        this.logEntry('debug', message, ...optionalParams);
    }
    info(message, ...optionalParams) {
        this.logEntry('info', message, ...optionalParams);
    }
    warn(message, ...optionalParams) {
        this.logEntry('warn', message, ...optionalParams);
    }
    error(message, ...optionalParams) {
        this.logEntry('error', message, ...optionalParams);
    }
}
exports.ConsoleLogger = ConsoleLogger;
class PluginLogger {
    logEntry(level, message, ...optionalParams) {
        const logEntry = {
            '@level': level,
        };
        if (message) {
            logEntry['@message'] = message;
        }
        if (optionalParams) {
            for (let n = 0; n < optionalParams.length; n += 2) {
                const key = optionalParams[n];
                const value = optionalParams[n + 1];
                if (key !== null && value !== null) {
                    logEntry[key] = value;
                }
            }
        }
        console.error(JSON.stringify(logEntry));
    }
    debug(message, ...optionalParams) {
        this.logEntry('debug', message, ...optionalParams);
    }
    info(message, ...optionalParams) {
        this.logEntry('info', message, ...optionalParams);
    }
    warn(message, ...optionalParams) {
        this.logEntry('warn', message, ...optionalParams);
    }
    error(message, ...optionalParams) {
        this.logEntry('error', message, ...optionalParams);
    }
}
exports.PluginLogger = PluginLogger;
//# sourceMappingURL=logger.js.map